@@ -11,7 +11,6 @@ from tempfile import mkdtemp
 import xml.etree.ElementTree as ET
 
 import yaml
-
 import requests
 from homura import download
 from shapely.geometry import Polygon
@@ -28,10 +27,14 @@ except TypeError:
         'SUSPECT_WORDS',
         failobj=join(dirname(abspath(__file__)), 'suspect_words.yaml')
         )
-WORDS = yaml.load(open(SUSPECT_WORDS_FILE, 'r').read())
+WORDS = yaml.safe_load(open(SUSPECT_WORDS_FILE, 'r').read())
 OSM_USERS_API = environ.get(
     'OSM_USERS_API',
-    'https://osm-comments-api.mapbox.com/api/v1/users/name/{username}'
+    'https://www.openstreetmap.org/api/0.6/user/{user_id}'
+    )
+MAPBOX_USERS_API = environ.get(
+    'MAPBOX_USERS_API',
+    'https://osm-comments-api.mapbox.com/api/v1/users/id/{user_id}?extra=true'
     )
 
 
@@ -39,13 +42,50 @@ class InvalidChangesetError(Exception):
     pass
 
 
+def get_user_details(user_id):
+    """Get information about number of changesets, blocks and mapping days of a
+    user, using both the OSM API and the Mapbox comments APIself.
+    """
+    reasons = []
+    try:
+        url = OSM_USERS_API.format(user_id=requests.compat.quote(user_id))
+        user_request = requests.get(url)
+        if user_request.status_code == 200:
+            user_data = user_request.content
+            xml_data = ET.fromstring(user_data)[0]
+            changesets = [i for i in xml_data if i.tag == 'changesets'][0]
+            blocks = [i for i in xml_data if i.tag == 'blocks'][0]
+            if int(changesets.get('count')) <= 5:
+                reasons.append('New mapper')
+            elif int(changesets.get('count')) <= 30:
+                url = MAPBOX_USERS_API.format(
+                    user_id=requests.compat.quote(user_id)
+                    )
+                user_request = requests.get(url)
+                if user_request.status_code == 200:
+                    mapping_days = int(
+                        user_request.json().get('extra').get('mapping_days')
+                        )
+                    if mapping_days <= 5:
+                        reasons.append('New mapper')
+            if int(blocks[0].get('count')) > 1:
+                reasons.append('User has multiple blocks')
+    except Exception as e:
+        message = 'Could not verify user of the changeset: {}, {}'
+        print(message.format(user_id, str(e)))
+    return reasons
+
+
 def changeset_info(changeset):
     """Return a dictionary with id, user, user_id, bounds, date of creation
     and all the tags of the changeset.
+
+    Args:
+        changeset: the XML string of the changeset.
     """
-    keys = [tag.attrib.get('k') for tag in changeset.getchildren()]
+    keys = [tag.attrib.get('k') for tag in changeset]
     keys += ['id', 'user', 'uid', 'bbox', 'created_at']
-    values = [tag.attrib.get('v') for tag in changeset.getchildren()]
+    values = [tag.attrib.get('v') for tag in changeset]
     values += [
         changeset.get('id'), changeset.get('user'), changeset.get('uid'),
         get_bounds(changeset), changeset.get('created_at')
@@ -57,8 +97,11 @@ def changeset_info(changeset):
 def get_changeset(changeset):
     """Get the changeset using the OSM API and return the content as a XML
     ElementTree.
+
+    Args:
+        changeset: the id of the changeset.
     """
-    url = 'http://www.openstreetmap.org/api/0.6/changeset/{}/download'.format(
+    url = 'https://www.openstreetmap.org/api/0.6/changeset/{}/download'.format(
         changeset
         )
     return ET.fromstring(requests.get(url).content)
@@ -67,15 +110,22 @@ def get_changeset(changeset):
 def get_metadata(changeset):
     """Get the metadata of a changeset using the OSM API and return it as a XML
     ElementTree.
+
+    Args:
+        changeset: the id of the changeset.
     """
-    url = 'http://www.openstreetmap.org/api/0.6/changeset/{}'.format(changeset)
-    return ET.fromstring(requests.get(url).content).getchildren()[0]
+    url = 'https://www.openstreetmap.org/api/0.6/changeset/{}'.format(changeset)
+    return ET.fromstring(requests.get(url).content)[0]
 
 
 def get_bounds(changeset):
     """Get the bounds of the changeset and return it as a Polygon object. If
     the changeset has not coordinates (case of the changesets that deal only
-    with relations), it returns an empty Polygon."""
+    with relations), it returns an empty Polygon.
+
+    Args:
+        changeset: the XML string of the changeset.
+    """
     try:
         return Polygon([
             (float(changeset.get('min_lon')), float(changeset.get('min_lat'))),
@@ -89,25 +139,38 @@ def get_bounds(changeset):
 
 
 def make_regex(words):
-    """Concatenate a list of words in a regular expression that detects any
-    word that starts with some of the words in a text.
+    """Concatenate a list of words in a regular expression. The regex is made to
+    check if a text has words that starts with any word in the list.
+
+    Args:
+        words: a list or tuple of strings,
     """
     return r'|'.join(
-        ["^{word}\.*|\.* {word}\.*".format(word=word) for word in words]
+        [r"^{word}\.*|\.* {word}\.*".format(word=word) for word in words]
         )
 
 
 def find_words(text, suspect_words, excluded_words=[]):
     """Check if a text has some of the suspect words (or words that starts with
-    one of the suspect words). You can set words to be excluded of the search,
-    so you can remove false positives like 'important' be detected when you
-    search by 'import'. Return True if the number of suspect words found is
-    greater than the number of excluded words.
+    one of the suspect words). You can set some words to be excluded of the
+    search, so you can remove false positives like 'important' be detected when
+    you search by 'import'. It will return True if the number of suspect words
+    found is greater than the number of excluded words. Otherwise, it will
+    return False.
+
+    Args:
+        text (str): a string with the text to be analysed. It will be converted
+            to lowercase.
+        suspect_words: a list of strings that you want to check the presence in
+            the text.
+        excluded_words: a list of strings to be whitelisted.
     """
     text = text.lower()
     suspect_found = [i for i in re.finditer(make_regex(suspect_words), text)]
     if len(excluded_words) > 0:
-        excluded_found = [i for i in re.finditer(make_regex(excluded_words), text)]
+        excluded_found = [
+            i for i in re.finditer(make_regex(excluded_words), text)
+            ]
         if len(suspect_found) > len(excluded_found):
             return True
         else:
@@ -121,17 +184,29 @@ def find_words(text, suspect_words, excluded_words=[]):
 
 class ChangesetList(object):
     """Read replication changeset file and return a list with the XML data of
-    each changeset. You can filter the changesets by passing a geojson file with
-    a Polygon of your area of interest.
+    each changeset. You can filter the changesets by passing a geojson file
+    with a Polygon of your area of interest.
     """
 
     def __init__(self, changeset_file, geojson=None):
+        """Read the changeset replication file, filter it you define a polygon
+        with your area of interest in a geojson file and define the .changesets
+        with the data of all changesets included in the replication file.
+
+        Args:
+            changeset_file (str): it can be the URL of a replication file in
+                https://planet.openstreetmap.org/replication/changesets/ or the
+                path to a local replication file.
+            geojson (str): path to a local geojson file containing a polygon.
+                The area of the polygon will be used to filter the changesets,
+                returning only the ones that intersect with it.
+        """
         self.read_file(changeset_file)
         if geojson:
             self.get_area(geojson)
             self.filter()
         else:
-            self.content = self.xml.getchildren()
+            self.content = self.xml
         self.changesets = [changeset_info(ch) for ch in self.content]
 
     def read_file(self, changeset_file):
@@ -156,13 +231,15 @@ class ChangesetList(object):
         object.
         """
         geojson = json.load(open(geojson, 'r'))
-        self.area = Polygon(geojson['features'][0]['geometry']['coordinates'][0])
+        self.area = Polygon(
+            geojson['features'][0]['geometry']['coordinates'][0]
+            )
 
     def filter(self):
         """Filter the changesets that intersects with the geojson geometry."""
         self.content = [
             ch
-            for ch in self.xml.getchildren()
+            for ch in self.xml
             if get_bounds(ch).intersects(self.area)
             ]
 
@@ -174,8 +251,7 @@ class Analyse(object):
             suspect_words=WORDS['common'] + WORDS['sources'],
             illegal_sources=WORDS['sources'], excluded_words=WORDS['exclude']):
         if type(changeset) in [int, str]:
-            changeset_details = changeset_info(get_metadata(changeset))
-            self.set_fields(changeset_details)
+            self.set_fields(changeset_info(get_metadata(changeset)))
         elif type(changeset) == dict:
             self.set_fields(changeset)
         else:
@@ -192,6 +268,21 @@ class Analyse(object):
         self.excluded_words = excluded_words
         self.illegal_sources = illegal_sources
         self.suspect_words = suspect_words
+        self.enabled_warnings = {
+            'warnings:almost_junction': 'Almost junction',
+            'warnings:close_nodes': 'Very close points',
+            'warnings:crossing_ways': 'Crossing ways',
+            'warnings:disconnected_way': 'Disconnected way',
+            'warnings:generic_name': 'Generic name',
+            'warnings:impossible_oneway': 'Impossible oneway',
+            'warnings:incompatible_source': 'suspect_word',
+            'warnings:missing_role': 'Missing role',
+            'warnings:missing_tag': 'Missing tag',
+            'warnings:outdated_tags': 'Outdated tags',
+            'warnings:private_data': 'Private information',
+            'warnings:tag_suggests_area': 'Line tagged as area',
+            'warnings:unsquare_way': 'Unsquare corners',
+            }
 
     def set_fields(self, changeset):
         """Set the fields of this class with the metadata of the analysed
@@ -201,9 +292,9 @@ class Analyse(object):
         self.user = changeset.get('user')
         self.uid = changeset.get('uid')
         self.editor = changeset.get('created_by', None)
+        self.review_requested = changeset.get('review_requested', False)
         self.host = changeset.get('host', 'Not reported')
         self.bbox = changeset.get('bbox').wkt
-        self.area = changeset.get('bbox').area
         self.comment = changeset.get('comment', 'Not reported')
         self.source = changeset.get('source', 'Not reported')
         self.imagery_used = changeset.get('imagery_used', 'Not reported')
@@ -214,44 +305,36 @@ class Analyse(object):
         self.suspicion_reasons = []
         self.is_suspect = False
         self.powerfull_editor = False
+        self.warning_tags = [
+            i for i in changeset.keys() if i.startswith('warnings:')
+            ]
+
+    def label_suspicious(self, reason):
+        """Add suspicion reason and set the suspicious flag."""
+        self.suspicion_reasons.append(reason)
+        self.is_suspect = True
 
     def full_analysis(self):
         """Execute the count and verify_words methods."""
         self.count()
         self.verify_words()
-        self.changeset_by_new_mapper()
+        self.verify_user()
+        self.verify_warning_tags()
 
-    def changeset_by_new_mapper(self):
+        if self.review_requested == 'yes':
+            self.label_suspicious('Review requested')
+
+    def verify_warning_tags(self):
+        for tag in self.warning_tags:
+            if tag in self.enabled_warnings.keys():
+                self.label_suspicious(self.enabled_warnings.get(tag))
+
+    def verify_user(self):
         """Verify if the changeset was made by a inexperienced mapper (anyone
-        with less than 5 edits).
+        with less than 5 edits) or by a user that was blocked more than once.
         """
-        def label_suspicious(self):
-            """Add suspicion reason and set the suspicious flag."""
-            reason = 'New mapper'
-            self.suspicion_reasons.append(reason)
-            self.is_suspect = True
-            return self
-
-        try:
-            # Convert username to ASCII and quote any special characters.
-            url = OSM_USERS_API.format(
-                username=requests.compat.quote(self.user)
-                )
-            r = requests.get(url)
-            if r.status_code == 404:
-                label_suspicious(self)
-            else:
-                # .decode is necessary to avoid failing in python 3.4 and 3.5
-                user_details = json.loads(
-                    requests.get(url).content.decode('utf-8')
-                    )
-                if user_details['changeset_count'] <= 5:
-                    label_suspicious(self)
-
-        except Exception as e:
-            print(
-                'changeset_by_new_mapper failed for: {}, {}'.format(self.id, str(e))
-                )
+        user_reasons = get_user_details(self.uid)
+        [self.label_suspicious(reason) for reason in user_reasons]
 
     def verify_words(self):
         """Verify the fields source, imagery_used and comment of the changeset
@@ -259,21 +342,21 @@ class Analyse(object):
         """
         if self.comment:
             if find_words(self.comment, self.suspect_words, self.excluded_words):
-                self.is_suspect = True
-                self.suspicion_reasons.append('suspect_word')
+                self.label_suspicious('suspect_word')
 
         if self.source:
             for word in self.illegal_sources:
                 if word in self.source.lower():
-                    self.is_suspect = True
-                    self.suspicion_reasons.append('suspect_word')
-                    break
+                    if word == 'yandex' and 'yandex panorama' in self.source.lower():
+                        pass
+                    else:
+                        self.label_suspicious('suspect_word')
+                        break
 
         if self.imagery_used:
             for word in self.illegal_sources:
                 if word in self.imagery_used.lower():
-                    self.is_suspect = True
-                    self.suspicion_reasons.append('suspect_word')
+                    self.label_suspicious('suspect_word')
                     break
 
         self.suspicion_reasons = list(set(self.suspicion_reasons))
@@ -281,26 +364,36 @@ class Analyse(object):
     def verify_editor(self):
         """Verify if the software used in the changeset is a powerfull_editor.
         """
+        powerful_editors = [
+            'josm', 'level0', 'merkaartor', 'qgis', 'arcgis', 'upload.py',
+            'osmapi', 'Services_OpenStreetMap'
+            ]
         if self.editor is not None:
-            for editor in ['josm', 'level0', 'merkaartor', 'qgis', 'arcgis']:
+            for editor in powerful_editors:
                 if editor in self.editor.lower():
                     self.powerfull_editor = True
                     break
 
             if 'iD' in self.editor:
                 trusted_hosts = [
-                    'http://www.openstreetmap.org/id',
-                    'https://www.openstreetmap.org/id',
-                    'http://improveosm.org/',
-                    'https://strava.github.io/iD/'
+                    'www.openstreetmap.org',
+                    'improveosm.org',
+                    'strava.github.io',
+                    'preview.ideditor.com',
+                    'hey.mapbox.com',
+                    'projets.pavie.info',
+                    'maps.mapcat.com',
+                    'id.softek.ir',
+                    'mapwith.ai',
+                    'tm4.hotosm.org',
+                    'tasks-stage.hotosm.org',
+                    'tasks.hotosm.org',
                     ]
-                if self.host not in trusted_hosts:
-                    self.is_suspect = True
-                    self.suspicion_reasons.append('Unknown iD instance')
+                if self.host.split('://')[-1].split('/')[0] not in trusted_hosts:
+                    self.label_suspicious('Unknown iD instance')
         else:
-            self.is_suspect = True
             self.powerfull_editor = True
-            self.suspicion_reasons.append('Software editor was not declared')
+            self.label_suspicious('Software editor was not declared')
 
     def count(self):
         """Count the number of elements created, modified and deleted by the
@@ -308,7 +401,7 @@ class Analyse(object):
         a mass deletion.
         """
         xml = get_changeset(self.id)
-        actions = [action.tag for action in xml.getchildren()]
+        actions = [action.tag for action in xml]
         self.create = actions.count('create')
         self.modify = actions.count('modify')
         self.delete = actions.count('delete')
@@ -318,17 +411,14 @@ class Analyse(object):
             if (self.create / len(actions) > self.percentage and
                     self.create > self.create_threshold and
                     (self.powerfull_editor or self.create > self.top_threshold)):
-                self.is_suspect = True
-                self.suspicion_reasons.append('possible import')
+                self.label_suspicious('possible import')
             elif (self.modify / len(actions) > self.percentage and
                     self.modify > self.modify_threshold):
-                self.is_suspect = True
-                self.suspicion_reasons.append('mass modification')
+                self.label_suspicious('mass modification')
             elif ((self.delete / len(actions) > self.percentage and
                     self.delete > self.delete_threshold) or
                     self.delete > self.top_threshold):
-                self.is_suspect = True
-                self.suspicion_reasons.append('mass deletion')
+                self.label_suspicious('mass deletion')
         except ZeroDivisionError:
             print('It seems this changeset was redacted')
 
@@ -341,7 +431,8 @@ class Analyse(object):
         fields_to_remove = [
             'create_threshold', 'modify_threshold', 'illegal_sources',
             'delete_threshold', 'percentage', 'top_threshold', 'suspect_words',
-            'excluded_words', 'host'
+            'excluded_words', 'host', 'review_requested', 'warning_tags',
+            'enabled_warnings'
             ]
         for field in fields_to_remove:
             ch_dict.pop(field)
