@@ -1,11 +1,9 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals
+from datetime import datetime
 from pytest import raises
 from shapely.geometry import Polygon
 
-from datetime import datetime
-import responses
-
 from osmcha.changeset import ChangesetList
 from osmcha.changeset import Analyse
 from osmcha.changeset import WORDS
@@ -25,6 +23,11 @@ def test_find_words():
     assert find_words('GooGle is not important', suspect_words, excluded_words)
     assert not find_words('somewhere in the world', suspect_words, excluded_words)
     assert find_words('дані по імпорту', suspect_words, excluded_words)
+    assert find_words('places from яндекс', suspect_words, excluded_words)
+    assert find_words('places from 2gis', suspect_words, excluded_words)
+    assert find_words('places from 2гис', suspect_words, excluded_words)
+    assert find_words('places from yandex', suspect_words, excluded_words)
+    assert not find_words('Yandex Panorama', suspect_words, excluded_words)
 
 
 def test_changeset_list():
@@ -78,7 +81,28 @@ def test_analyse_init():
     assert ch.user == 'JustTest'
     assert ch.uid == '123123'
     assert ch.date == datetime(2015, 4, 25, 18, 8, 46)
-    assert ch.area == 0.00035454780569997915
+
+
+def test_analyse_label_suspicious():
+    ch_dict = {
+        'created_by': 'Potlatch 2',
+        'created_at': '2015-04-25T18:08:46Z',
+        'build': '2.3-650-gad99430',
+        'version': '2.3',
+        'comment': 'Put data from Google',
+        'id': '1',
+        'user': 'JustTest',
+        'uid': '123123',
+        'bbox': Polygon([
+            (-71.0646843, 44.2371354), (-71.0048652, 44.2371354),
+            (-71.0048652, 44.2430624), (-71.0646843, 44.2430624),
+            (-71.0646843, 44.2371354)
+            ])
+        }
+    ch = Analyse(ch_dict)
+    ch.label_suspicious('some reason')
+    assert 'some reason' in ch.suspicion_reasons
+    assert ch.is_suspect
 
 
 def test_changeset_without_coords():
@@ -168,6 +192,26 @@ def test_analyse_verify_words():
     ch.verify_words()
     assert not ch.is_suspect
 
+    ch_dict = {
+        'created_by': 'Potlatch 2',
+        'created_at': '2015-04-25T18:08:46Z',
+        'build': '2.3-650-gad99430',
+        'version': '2.3',
+        'comment': 'Somewhere in Brazil',
+        'source': 'Yandex Panorama',
+        'id': '1',
+        'user': 'JustTest',
+        'uid': '123123',
+        'bbox': Polygon([
+            (-71.0646843, 44.2371354), (-71.0048652, 44.2371354),
+            (-71.0048652, 44.2430624), (-71.0646843, 44.2430624),
+            (-71.0646843, 44.2371354)
+            ])
+        }
+    ch = Analyse(ch_dict)
+    ch.verify_words()
+    assert not ch.is_suspect
+
 
 def test_analyse_verify_editor_josm():
     """Test if JOSM is a powerfull_editor."""
@@ -250,12 +294,12 @@ def test_analyse_verify_editor_qgis():
 
 
 def test_analyse_verify_editor_id_osm():
-    """Test if iD is not a powerfull_editor and if https://www.openstreetmap.org/id
+    """Test if iD is not a powerfull_editor and if https://www.openstreetmap.org/edit
     is a trusted instance.
     """
     ch_dict = {
         'created_by': 'iD 1.7.3',
-        'host': 'https://www.openstreetmap.org/id',
+        'host': 'https://www.openstreetmap.org/edit',
         'created_at': '2015-04-25T18:08:46Z',
         'comment': 'add pois',
         'id': '1',
@@ -297,7 +341,7 @@ def test_analyse_verify_editor_id_improveosm():
     assert ch.suspicion_reasons == []
 
 
-def test_analyse_verify_editor_id_improveosm():
+def test_analyse_verify_editor_id_strava():
     """Test if iD is not a powerfull_editor and if https://strava.github.io/iD/
     is a trusted instance.
     """
@@ -321,13 +365,57 @@ def test_analyse_verify_editor_id_improveosm():
     assert ch.suspicion_reasons == []
 
 
-def test_analyse_verify_editor_id_unknown_instance():
+def test_analyse_verify_editor_rapid():
+    """Test if RapiD is not a powerfull_editor and a trusted instance."""
+    ch_dict = {
+        'created_by': 'RapiD 0.9.0',
+        'host': 'https://mapwith.ai/rapid',
+        'created_at': '2015-04-25T18:08:46Z',
+        'comment': 'add pois',
+        'id': '1',
+        'user': 'JustTest',
+        'uid': '123123',
+        'bbox': Polygon([
+            (-71.0646843, 44.2371354), (-71.0048652, 44.2371354),
+            (-71.0048652, 44.2430624), (-71.0646843, 44.2430624),
+            (-71.0646843, 44.2371354)
+            ])
+        }
+    ch = Analyse(ch_dict)
+    ch.verify_editor()
+    assert ch.powerfull_editor is False
+    assert ch.suspicion_reasons == []
+
+
+def test_analyse_verify_editor_rapid_test():
+    """Test if RapiD test is not a powerfull_editor and a trusted instance."""
+    ch_dict = {
+        'created_by': 'RapiD 0.9.0',
+        'host': 'https://mapwith.ai/rapidtest',
+        'created_at': '2015-04-25T18:08:46Z',
+        'comment': 'add pois',
+        'id': '1',
+        'user': 'JustTest',
+        'uid': '123123',
+        'bbox': Polygon([
+            (-71.0646843, 44.2371354), (-71.0048652, 44.2371354),
+            (-71.0048652, 44.2430624), (-71.0646843, 44.2430624),
+            (-71.0646843, 44.2371354)
+            ])
+        }
+    ch = Analyse(ch_dict)
+    ch.verify_editor()
+    assert ch.powerfull_editor is False
+    assert ch.suspicion_reasons == []
+
+
+def test_verify_editor_id_unknown_instance():
     """Test if iD is not a powerfull_editor and if 'Unknown iD instance' is added
     to suspicion_reasons.
     """
     ch_dict = {
         'created_by': 'iD 1.7.3',
-        'host': 'http://anotherhost.com',
+        'host': 'http://anotherhost.com/iD',
         'created_at': '2015-04-25T18:08:46Z',
         'comment': 'add pois',
         'id': '1',
@@ -346,6 +434,75 @@ def test_analyse_verify_editor_id_unknown_instance():
     assert ch.is_suspect
 
 
+def test_verify_editor_id_is_known_instance():
+    """Test if iD is not a powerfull_editor and if 'Unknown iD instance' is added
+    to suspicion_reasons.
+    """
+    ch_dict = {
+        'created_by': 'iD 1.7.3',
+        'host': 'https://www.openstreetmap.org/iD',
+        'created_at': '2015-04-25T18:08:46Z',
+        'comment': 'add pois',
+        'id': '1',
+        'user': 'JustTest',
+        'uid': '123123',
+        'bbox': Polygon([
+            (-71.0646843, 44.2371354), (-71.0048652, 44.2371354),
+            (-71.0048652, 44.2430624), (-71.0646843, 44.2430624),
+            (-71.0646843, 44.2371354)
+            ])
+        }
+    ch = Analyse(ch_dict)
+    ch.verify_editor()
+    assert ch.powerfull_editor is False
+    assert 'Unknown iD instance' not in ch.suspicion_reasons
+    assert ch.is_suspect is False
+
+
+def test_verify_hotosm_id_is_known_instance():
+    """Test if iD is not a powerfull_editor and if 'Unknown iD instance' is added
+    to suspicion_reasons.
+    """
+    ch1 = {
+        'created_by': 'iD 1.7.3',
+        'host': 'https://tm4.hotosm.org/projects/23/map/',
+        'created_at': '2015-04-25T18:08:46Z',
+        'comment': 'add pois',
+        'id': '1',
+        'user': 'JustTest',
+        'uid': '123123',
+        'bbox': Polygon([
+            (-71.0646843, 44.2371354), (-71.0048652, 44.2371354),
+            (-71.0048652, 44.2430624), (-71.0646843, 44.2430624),
+            (-71.0646843, 44.2371354)
+            ])
+        }
+    ch2 = {
+        'created_by': 'iD 1.7.3',
+        'host': 'https://tasks.hotosm.org/projects/23/map/',
+        'created_at': '2015-04-25T18:08:46Z',
+        'comment': 'add pois',
+        'id': '1',
+        'user': 'JustTest',
+        'uid': '123123',
+        'bbox': Polygon([
+            (-71.0646843, 44.2371354), (-71.0048652, 44.2371354),
+            (-71.0048652, 44.2430624), (-71.0646843, 44.2430624),
+            (-71.0646843, 44.2371354)
+            ])
+        }
+    ch = Analyse(ch1)
+    ch.verify_editor()
+    assert ch.powerfull_editor is False
+    assert 'Unknown iD instance' not in ch.suspicion_reasons
+    assert ch.is_suspect is False
+    ch_2 = Analyse(ch2)
+    ch_2.verify_editor()
+    assert ch_2.powerfull_editor is False
+    assert 'Unknown iD instance' not in ch_2.suspicion_reasons
+    assert ch_2.is_suspect is False
+
+
 def test_analyse_verify_editor_Potlatch2():
     """Test if Potlatch 2 is not a powerfull_editor."""
     ch_dict = {
@@ -481,7 +638,6 @@ def test_get_dict():
     assert 'uid' in ch.get_dict().keys()
     assert 'editor' in ch.get_dict().keys()
     assert 'bbox' in ch.get_dict().keys()
-    assert 'area' in ch.get_dict().keys()
     assert 'date' in ch.get_dict().keys()
     assert 'comment' in ch.get_dict().keys()
     assert 'source' in ch.get_dict().keys()
@@ -492,7 +648,27 @@ def test_get_dict():
     assert 'create' in ch.get_dict().keys()
     assert 'modify' in ch.get_dict().keys()
     assert 'delete' in ch.get_dict().keys()
-    assert len(ch.get_dict().keys()) == 16
+    assert len(ch.get_dict().keys()) == 15
+
+    # An iD changeset with warnings:
+    ch = Analyse(72783703)
+    ch.full_analysis()
+    assert 'id' in ch.get_dict().keys()
+    assert 'user' in ch.get_dict().keys()
+    assert 'uid' in ch.get_dict().keys()
+    assert 'editor' in ch.get_dict().keys()
+    assert 'bbox' in ch.get_dict().keys()
+    assert 'date' in ch.get_dict().keys()
+    assert 'comment' in ch.get_dict().keys()
+    assert 'source' in ch.get_dict().keys()
+    assert 'imagery_used' in ch.get_dict().keys()
+    assert 'is_suspect' in ch.get_dict().keys()
+    assert 'powerfull_editor' in ch.get_dict().keys()
+    assert 'suspicion_reasons' in ch.get_dict().keys()
+    assert 'create' in ch.get_dict().keys()
+    assert 'modify' in ch.get_dict().keys()
+    assert 'delete' in ch.get_dict().keys()
+    assert len(ch.get_dict().keys()) == 15
 
     # A JOSM changeset
     ch = Analyse(46315321)
@@ -502,7 +678,6 @@ def test_get_dict():
     assert 'uid' in ch.get_dict().keys()
     assert 'editor' in ch.get_dict().keys()
     assert 'bbox' in ch.get_dict().keys()
-    assert 'area' in ch.get_dict().keys()
     assert 'date' in ch.get_dict().keys()
     assert 'comment' in ch.get_dict().keys()
     assert 'source' in ch.get_dict().keys()
@@ -513,7 +688,7 @@ def test_get_dict():
     assert 'create' in ch.get_dict().keys()
     assert 'modify' in ch.get_dict().keys()
     assert 'delete' in ch.get_dict().keys()
-    assert len(ch.get_dict().keys()) == 16
+    assert len(ch.get_dict().keys()) == 15
 
 
 def test_changeset_without_tags():
@@ -524,6 +699,13 @@ def test_changeset_without_tags():
     assert 'Software editor was not declared' in ch.suspicion_reasons
 
 
+def test_changeset_by_user_with_more_than_one_block():
+    changeset = Analyse(34879408)
+    changeset.full_analysis()
+    assert 'User has multiple blocks' in changeset.suspicion_reasons
+    assert changeset.is_suspect
+
+
 def test_changeset_by_new_mapper():
     changeset = Analyse(46756461)
     changeset.full_analysis()
@@ -531,6 +713,21 @@ def test_changeset_by_new_mapper():
     assert changeset.is_suspect
 
 
+def test_changeset_by_new_mapper():
+    changeset = Analyse(36700893)
+    changeset.full_analysis()
+    print(changeset.suspicion_reasons)
+    assert 'New mapper' in changeset.suspicion_reasons
+    assert changeset.is_suspect
+
+
+def test_changeset_with_6_mapping_days():
+    changeset = Analyse(13523366)
+    changeset.full_analysis()
+    assert 'New mapper' not in changeset.suspicion_reasons
+    assert not changeset.is_suspect
+
+
 def test_changeset_by_old_mapper_with_unicode_username():
     changeset = Analyse(46790192)
     changeset.full_analysis()
@@ -545,33 +742,172 @@ def test_changeset_by_old_mapper_with_special_character_username():
     assert not changeset.is_suspect
 
 
-@responses.activate
-def test_changeset_by_mapper_who_does_not_exist():
-    changeset_meta_data = '''<osm version="0.6" generator="CGImap 0.5.8 (15904 thorn-02.openstreetmap.org)" copyright="OpenStreetMap and contributors" attribution="http://www.openstreetmap.org/copyright" license="http://opendatacommons.org/licenses/odbl/1-0/"><changeset id="44469157" created_at="2016-12-17T12:37:41Z" closed_at="2016-12-17T12:37:43Z" open="false" user="bkowshik" uid="1087876" min_lat="12.9316906" min_lon="77.5555034" max_lat="12.9316906" max_lon="77.5555034" comments_count="0"><tag k="comment" v="Added a hotel"/><tag k="locale" v="en-US"/><tag k="host" v="http://www.openstreetmap.org/id"/><tag k="imagery_used" v="Bing aerial imagery"/><tag k="created_by" v="iD 2.0.1"/></changeset></osm>'''
-    changeset_data = '''<osmChange version="0.6" generator="OpenStreetMap server" copyright="OpenStreetMap and contributors" attribution="http://www.openstreetmap.org/copyright" license="http://opendatacommons.org/licenses/odbl/1-0/"><create><node id="4558466455" changeset="44469157" timestamp="2016-12-17T12:37:43Z" version="1" visible="true" user="bkowshik" uid="1087876" lat="12.9316906" lon="77.5555034"><tag k="fast_food" v="regional"/><tag k="food" v="veg"/><tag k="name" v="Sankethi's"/><tag k="operator" v="Sankethi Ventures"/><tag k="smoking" v="no"/><tag k="tourism" v="hotel"/></node></create></osmChange>'''
-
-    # Mock changeset meta data request.
-    responses.add(
-        responses.GET,
-        'http://www.openstreetmap.org/api/0.6/changeset/44469157',
-        body=changeset_meta_data,
-        status=200
-    )
-    # Mock changeset data request.
-    responses.add(
-        responses.GET,
-        'http://www.openstreetmap.org/api/0.6/changeset/44469157/download',
-        body=changeset_data,
-        status=200
-    )
-    # Mock download user details request.
-    responses.add(
-        responses.GET,
-        'https://osm-comments-api.mapbox.com/api/v1/users/name/bkowshik',
-        json={"error": "not found"},
-        status=404  # To denote that the user does not exist.
-        )
-    changeset = Analyse(44469157)
+def test_changeset_with_review_requested():
+    ch_dict = {
+        'created_by': 'Potlatch 2',
+        'created_at': '2015-04-25T18:08:46Z',
+        'comment': 'add pois',
+        'id': '1',
+        'user': 'JustTest',
+        'uid': '123123',
+        'review_requested': 'yes',
+        'bbox': Polygon([
+            (-71.0646843, 44.2371354), (-71.0048652, 44.2371354),
+            (-71.0048652, 44.2430624), (-71.0646843, 44.2430624),
+            (-71.0646843, 44.2371354)
+            ])
+        }
+    changeset = Analyse(ch_dict)
     changeset.full_analysis()
-    assert 'New mapper' in changeset.suspicion_reasons
+    assert 'Review requested' in changeset.suspicion_reasons
     assert changeset.is_suspect
+
+
+def test_changeset_with_warning_tag_almost_junction():
+    ch_dict = {
+        'created_by': 'iD',
+        'created_at': '2019-04-25T18:08:46Z',
+        'host': 'https://www.openstreetmap.org/edit',
+        'comment': 'add pois',
+        'id': '1',
+        'user': 'JustTest',
+        'uid': '123123',
+        'warnings:almost_junction': '1',
+        'warnings:missing_role': '1',
+        'warnings:missing_tag': '1',
+        'warnings:private_data': '1',
+        'warnings:tag_suggests_area': '1',
+        'warnings:unsquare_way': '1',
+        'bbox': Polygon([
+            (-71.0646843, 44.2371354), (-71.0048652, 44.2371354),
+            (-71.0048652, 44.2430624), (-71.0646843, 44.2430624),
+            (-71.0646843, 44.2371354)
+            ])
+        }
+    changeset = Analyse(ch_dict)
+    changeset.full_analysis()
+    assert 'Almost junction' in changeset.suspicion_reasons
+    assert 'Missing role' in changeset.suspicion_reasons
+    assert 'Missing tag' in changeset.suspicion_reasons
+    assert 'Private information' in changeset.suspicion_reasons
+    assert 'Line tagged as area' in changeset.suspicion_reasons
+    assert 'Unsquare corners' in changeset.suspicion_reasons
+    assert changeset.is_suspect
+
+
+def test_changeset_with_warning_tag_close_nodes():
+    ch_dict = {
+        'created_by': 'iD',
+        'created_at': '2019-04-25T18:08:46Z',
+        'host': 'https://www.openstreetmap.org/edit',
+        'comment': 'add pois',
+        'id': '1',
+        'user': 'JustTest',
+        'uid': '123123',
+        'warnings:close_nodes': '1',
+        'bbox': Polygon([
+            (-71.0646843, 44.2371354), (-71.0048652, 44.2371354),
+            (-71.0048652, 44.2430624), (-71.0646843, 44.2430624),
+            (-71.0646843, 44.2371354)
+            ])
+        }
+    changeset = Analyse(ch_dict)
+    changeset.full_analysis()
+    assert 'Very close points' in changeset.suspicion_reasons
+    assert changeset.is_suspect
+
+
+def test_changeset_with_warning_tag_crossing_ways():
+    ch_dict = {
+        'created_by': 'iD',
+        'created_at': '2019-04-25T18:08:46Z',
+        'host': 'https://www.openstreetmap.org/edit',
+        'comment': 'add pois',
+        'id': '1',
+        'user': 'JustTest',
+        'uid': '123123',
+        'warnings:crossing_ways': '1',
+        'bbox': Polygon([
+            (-71.0646843, 44.2371354), (-71.0048652, 44.2371354),
+            (-71.0048652, 44.2430624), (-71.0646843, 44.2430624),
+            (-71.0646843, 44.2371354)
+            ])
+        }
+    changeset = Analyse(ch_dict)
+    changeset.full_analysis()
+    assert 'Crossing ways' in changeset.suspicion_reasons
+    assert changeset.is_suspect
+
+
+def test_changeset_with_warning_tag_disconnected_way():
+    ch_dict = {
+        'created_by': 'iD',
+        'created_at': '2019-04-25T18:08:46Z',
+        'host': 'https://www.openstreetmap.org/edit',
+        'comment': 'add pois',
+        'id': '1',
+        'user': 'JustTest',
+        'uid': '123123',
+        'warnings:disconnected_way': '4',
+        'warnings:generic_name': '4',
+        'warnings:impossible_oneway': '4',
+        'warnings:incompatible_source': '4',
+        'warnings:outdated_tags': '9',
+        'bbox': Polygon([
+            (-71.0646843, 44.2371354), (-71.0048652, 44.2371354),
+            (-71.0048652, 44.2430624), (-71.0646843, 44.2430624),
+            (-71.0646843, 44.2371354)
+            ])
+        }
+    changeset = Analyse(ch_dict)
+    changeset.full_analysis()
+    assert 'Disconnected way' in changeset.suspicion_reasons
+    assert 'Generic name' in changeset.suspicion_reasons
+    assert 'Impossible oneway' in changeset.suspicion_reasons
+    assert 'suspect_word' in changeset.suspicion_reasons
+    assert 'Outdated tags' in changeset.suspicion_reasons
+    assert changeset.is_suspect
+
+
+def test_changeset_with_warning_tag_fix_me():
+    ch_dict = {
+        'created_by': 'iD',
+        'created_at': '2019-04-25T18:08:46Z',
+        'host': 'https://www.openstreetmap.org/edit',
+        'comment': 'add pois',
+        'id': '1',
+        'user': 'JustTest',
+        'uid': '123123',
+        'warnings:fix_me': '0',
+        'bbox': Polygon([
+            (-71.0646843, 44.2371354), (-71.0048652, 44.2371354),
+            (-71.0048652, 44.2430624), (-71.0646843, 44.2430624),
+            (-71.0646843, 44.2371354)
+            ])
+        }
+    changeset = Analyse(ch_dict)
+    changeset.full_analysis()
+    assert changeset.suspicion_reasons == []
+    assert not changeset.is_suspect
+
+
+def test_changeset_with_warning_tag_invalid_format():
+    ch_dict = {
+        'created_by': 'iD',
+        'created_at': '2019-04-25T18:08:46Z',
+        'host': 'https://www.openstreetmap.org/edit',
+        'comment': 'add pois',
+        'id': '1',
+        'user': 'JustTest',
+        'uid': '123123',
+        'warnings:invalid_format': '0',
+        'bbox': Polygon([
+            (-71.0646843, 44.2371354), (-71.0048652, 44.2371354),
+            (-71.0048652, 44.2430624), (-71.0646843, 44.2430624),
+            (-71.0646843, 44.2371354)
+            ])
+        }
+    changeset = Analyse(ch_dict)
+    changeset.full_analysis()
+    assert changeset.suspicion_reasons == []
+    assert not changeset.is_suspect
